#! /usr/bin/python

## commands.py
## Copyright (C) 2007 Ivan Uemlianin
## Contact: ivan@llaisdy.com
## 
## This file is part of Trefnydd.
##
## Trefnydd is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Trefnydd is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Trefnydd (i.e., doc/gpl.txt); if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
## or see http://www.gnu.org/licenses/gpl.txt


"""
========
trefnydd
========

Package:   trefnydd
Module:    lib.htk.commands
Copyright: 2007 Ivan A. Uemlianin
Contact:   ivan@llaisdy.com
Licence:   GNU GPL (see doc/gpl.txt)

commands
========

class HTKCommand(object)  and subclasses

"""

import os

# from trefnydd.gui import gui

class HTKCommand(object):
    def __init__(self, debug=False):
        self.debug = debug
        self.name = ''
        self.usage = ''
        self.session = None # session with configs, fns, dirns, etc
        self._switches = {}
        self.switch_keys = []
        self.argument = ''
        # self.log = '' # ?
        self.cmdout = ''
        self.cmderr = ''

    def get_switches(self):
        return self._switches

    def set_switches(self, switches):
        for k in switches:
            if k in self.switch_keys:
                self._switches[k] = switches[k]
            else:
                raise StandardError, '%s: %s not a permissible option' \
                      % (self.name, k)

    switches = property(get_switches, set_switches)

    def get_cmd(self): #, switches=[]):
        switches = ' '.join(['%s %s' % (s[0], s[1])
                             for s in self.switches.items()])
        cmd = '%s %s' % (self.name, switches)
        if self.argument: cmd = '%s %s' % (cmd, self.argument)
        return cmd

    cmd = property(get_cmd)

    def run(self):
        stdin, stdout, stderr = os.popen3(self.cmd)
        self.cmdout = stdout.read()
        self.cmderr = stderr.read()
        if self.debug:
            out = self.getLog()
            if out: print out

    def getLog(self):
        line = '-' * len(self.name)
        out = '%s\n%s\n%s\n\n' \
              % (line, self.name, line)
        out = '%s%s\n\n' % (out, self.pprintcmd(self.cmd))
        if self.cmdout: out = '%s%s' % (out, self.pprintout(self.cmdout))
        if self.cmderr: out = '%s%s' % (out, self.pprintout(self.cmderr, 'cmderr'))
        return out

    log = property(getLog)

    def pprintcmd(self, cmd):
        cmd = cmd.replace(' -', '\n\t-')
        lidx = cmd.find('-')
        ridx = cmd.rfind(' ')
        cmd = 'Parameters\n----------\n\n::\n\n\t%s\n\t%s' \
              % (cmd[lidx:ridx], cmd[ridx+1:])
        return cmd

    def pprintout(self, out, label='cmdout'):
        line = '-' * len(label)
        out = out.replace('\n', '\n\t')
        out = '%s\n%s\n\n::\n\n\t%s\n' \
              % (label, line, out)
        return out
    

class HCopy(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HCopy'
        self.usage = """USAGE: HCopy [options] src [ + src ...] tgt ...

 Option                                       Default

 -a i     Use level i labels                  1
 -e t     End copy at time t                  EOF
 -i mlf   Save labels to mlf s                null
 -l dir   Output target label files to dir    current
 -m t     Set margin of t around x/n segs     0
 -n i [j] Extract i'th [to j'th] label        off
 -s t     Start copy at time t                0
 -t n     Set trace line width to n           70
 -x s [n] Extract [n'th occ of] label  s      off
 -A      Print command line arguments         off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -F fmt  Set source data format to fmt        as config
 -G fmt  Set source label format to fmt       as config
 -I mlf  Load master label file mlf
 -L dir  Set input label (or net) dir         current
 -O      Set target data format to fmt        as config
 -P      Set target label format to fmt       as config
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off
 -X ext  Set input label (or net) file ext    lab

"""
        self.switch_keys = ['-a', '-e', '-i', '-l', '-m',
                            '-n', '-s', '-t', '-x',
                            '-A', '-C', '-D', '-F', '-G',
                            '-I', '-L', '-O', '-P', '-S',
                            '-T', '-V', '-X']


class HInit(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HInit'
        self.usage = """USAGE: HInit [options] hmmFile trainFiles...

 Option                                       Default

 -e f    Set convergence factor epsilon       1.0E-4
 -i N    Set max iterations to N              20
 -l s    Set segment label to s               none
 -m N    Set min segments needed              3
 -n      Update hmm (suppress uniform seg)    off
 -o fn   Store new hmm def in fn (name only)  outDir/srcfn
 -u mvwt Update m)eans v)ars w)ghts t)rans    mvwt
 -v f    Set minimum variance to f            1.0E-2
 -w f    set mix wt/disc prob floor to f      0.0
 -A      Print command line arguments         off
 -B      Save HMMs/transforms as binary       off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -F fmt  Set source data format to fmt        as config
 -G fmt  Set source label format to fmt       as config
 -H mmf  Load HMM macro file mmf
 -I mlf  Load master label file mlf
 -L dir  Set input label (or net) dir         current
 -M dir  Dir to write HMM macro files         current
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off
 -X ext  Set input label (or net) file ext    lab

"""
        self.switch_keys = ['-e', '-i', '-l', '-m', '-n',
                            '-o', '-u', '-v', '-w',
                            '-A', '-B', '-C', '-D', '-F',
                            '-G', '-H', '-I', '-L', '-M',
                            '-S', '-T', '-V', '-X']



class HRest(HTKCommand):
    def __init__(self, debug=False):
        HTKCommand.__init__(self, debug)
        self.name = 'HRest'
        self.usage = """USAGE: HRest [options] hmmFile trainFiles...

 Option                                       Default

 -e f    Set convergence factor epsilon       1.0E-4
 -i N    Set max iterations to N              20
 -l s    Set segment label to s               none
 -m N    Set min segments needed              3
 -t      Disable short segment rejection      on
 -u tmvw Update t)rans m)eans v)ars w)ghts    tmvw
 -v f    Set minimum variance to f            0.0
 -c f    Tied Mixture pruning threshold       10.0
 -w f    Set mix wt floor to f x MINMIX       0.0
 -A      Print command line arguments         off
 -B      Save HMMs/transforms as binary       off
 -C cf   Set config file to cf                default
 -D      Display configuration variables      off
 -F fmt  Set source data format to fmt        as config
 -G fmt  Set source label format to fmt       as config
 -H mmf  Load HMM macro file mmf
 -I mlf  Load master label file mlf
 -L dir  Set input label (or net) dir         current
 -M dir  Dir to write HMM macro files         current
 -S f    Set script file to f                 none
 -T N    Set trace flags to N                 0
 -V      Print version information            off
 -X ext  Set input label (or net) file ext    lab

"""
        self.switch_keys = ['-e', '-i', '-l', '-m', '-t', '-u', '-v', '-c', '-w', '-A', '-B', '-C', '-D', '-F', '-G', '-H', '-I', '-L', '-M', '-S', '-T', '-V', '-X']

        



def main():
    pass

if __name__ == '__main__':
    main()
 
