#! /usr/bin/python

## location.py
## Copyright (C) 2007 Ivan Uemlianin
## Contact: ivan@llaisdy.com
## 
## This file is part of Trefnydd.
##
## Trefnydd is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Trefnydd is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Trefnydd (i.e., doc/gpl.txt); if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
## or see http://www.gnu.org/licenses/gpl.txt


"""
========
trefnydd
========

Package:   trefnydd
Module:    location
Copyright: 2007 Ivan A. Uemlianin
Contact:   ivan@llaisdy.com
Licence:   GNU GPL (see doc/gpl.txt)

location
========

"""


# from trefnydd.gui import gui

import os

import file_IO

class Located(object):
    def __init__(self, debug=False):
        self.debug = debug
        self._homeDir = '.'
        self.changed = False
        self.name =''

    def getHomeDir(self):
        if not self._homeDir:
            raise StandardError, 'No home directory defined.'
        return self._homeDir

    def setHomeDir(self, dirn):
        if dirn != self._homeDir:
            self._homeDir = file_IO.getDir(os.path.abspath(dirn))
            self.changed = True

    homeDir = property(getHomeDir, setHomeDir)

class DataSetContainer(Located):
    def __init__(self):
        Located.__init__(self)
        self.ext = ''
        self.name = ''
        self.comment = ''
        self.priorityFunc = None

    def getPreamble(self):
        return """# trefnydd::DataSetContainer\n
priorityFunc = %s
comment =      %s\n\n""" % (self.priorityFunc, self.comment)

    preamble = property(getPreamble)

    def save(self, fn=''):
        if not fn:
            fn = '%s%s' % (self.name, self.ext)
        fullFn = os.path.join(self.homeDir, fn)
        body = '\n'.join([i.__repr__() for i in self])
        out = '%s%s\n' % (self.preamble.encode('utf-8'), body)
        open(fullFn, 'w').write(out.encode('utf-8'))

    def getLines(self, fn, enc='utf-8'):
        data = unicode(open(fn).read(), enc)
        dl = []
        for line in [i for i in data.split(u'\n')
                     if i.strip() and i[0] != u'#']:
            dl.append(line)
        return dl

    def loadDataSet(self, fn):
        self.name = os.path.splitext(os.path.basename(fn))[0]
        for line in self.getLines(fn):
##         # 300907: old version - delete when tested            
##         for line in [i for i in open(fn)
##                      if i.strip() and i[0] != '#']:
##         # end delete            
            # ignore blank lines and comments
            if '=' in line: # line is name = value
                name, value = [i.strip().lower()
                               for i in line.split('=')]
                self.loadNameValuePair(name, value)
            else:
                self.loadDataItem(line)
        self.changed = True

    def loadNameValuePair(self, name, value):
        pass

    def loadDataItem(self, line):
        pass

    def importDataFromFile(self, fn):
        self.loadDataSet(fn)

    

def main():
    pass

if __name__ == '__main__':
    main()
 
