#! /usr/bin/python

## stats.py
## Copyright (C) 2007 Ivan Uemlianin
## Contact: ivan@llaisdy.com
## 
## This file is part of Trefnydd.
##
## Trefnydd is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## Trefnydd is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Trefnydd (i.e., doc/gpl.txt); if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
## or see http://www.gnu.org/licenses/gpl.txt


"""
========
trefnydd
========

Package:   trefnydd
Module:    stats
Copyright: 2007 Ivan A. Uemlianin
Contact:   ivan@llaisdy.com
Licence:   GNU GPL (see doc/gpl.txt)

stats
=====

"""


def mean(nlist):
    return reduce(lambda x,y : x+y, nlist) / len(nlist)

def median(nlist):
    return quantiles(2, nlist)[0]

def quartiles(nlist):
    return quantiles(4, nlist)

def quantiles(ord, nlist):
    quants = []
    nlist.sort()
    ln = len(nlist)
    step = (ln + 1 - ord)*1.0 / ord
    for i in float_range(step, ln, step+1):
        if i == i//1:
            q = nlist[int(i)]
        else:
            idx, rem = divmod(i, 1)
            idx = int(idx)
            lower, upper = nlist[idx:idx+2]
            delta = (upper - lower) * rem
            q = lower + delta
        quants.append(q)
    return quants

## From 'The Python Cookbook' (O'Reilly, 2002, eds: Martelli & Ascher)

def float_range(begin, end, step):
    i = 0 
    while 1:
        next = begin + i * step
        if step > 0 and next >= end:
            break
        elif step < 0 and next <= end:
            break
        yield next
        i += 1

def main():
    nls = [
        [66.0, 72.0, 79.0, 84.0, 102.0, 110.0, 123.0, 144.0, 162.0, 169.0, 414.0, 84.0, 84.0],
        [1,2,3,4,5,6,7,8,9,0,10]
        ]

    for nl in nls:
        print '==========\ndata: %s' % nl
        print 'quartiles: %s' % quantiles(4, nl)
        print 'median: %s' % median(nl)
        print 'mean: %s\n\n' % mean(nl)
    

if __name__ == '__main__':
    main()
    

#
